{ open Parser }

rule token = parse
  [' ' '\t' '\r' '\n'] { token lexbuf } (* Whitespace *)
| "/*"     { comment lexbuf }           (* Comments *)
| '('      { LPAREN }
| ')'      { RPAREN }
| '{'      { LBRACE }
| '}'      { RBRACE }
| ';'      { SEMI }
| ','      { COMMA }
| '+'      { PLUS }
| '-'      { MINUS }
| '*'      { TIMES }
| '/'      { DIVIDE }
| '='      { ASSIGN }
| '^'      { CONCAT }
| "=="     { EQ }
| "!="     { NEQ }
| '<'      { LT }
| "<="     { LEQ }
| ">"      { GT }
| ">="     { GEQ }
| "||"     { OR }
| "&&"     { AND }
| "if"     { IF }
| "else"   { ELSE }
| "for"    { FOR }
| "while"  { WHILE }
| "return" { RETURN }
| "int"    { INT }
| "string" { STRING }
| "boolean" { BOOLEAN }
| "float"  { FLOAT }
| "true"   as lxm { LBOOL(bool_of_string lxm) }
| "false"  as lxm { LBOOL(bool_of_string lxm) }
| ['0'-'9']+['.']['0'-'9']+ as lxm { LFLOAT(float_of_string lxm) }
| ['0'-'9']+ as lxm { LINTEGER(int_of_string lxm) }
| ['"']['a'-'z' 'A'-'Z' '0'-'9' '_' ' ' '!' '@' '#' '$' '%' '^' '&' '*' '(' ')' '{' '}' '|' ';' '<' '>' '.' ',' '?' '/' '\n' '+' '-'  '=' '\\' ''' ]*['"'] as lxm { LSTRING(lxm) }
| ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_'  ]* as lxm { ID(lxm) }
| eof { EOF }
| _ as char { raise (Failure("illegal character " ^ Char.escaped char)) }

and comment = parse
  "*/" { token lexbuf }
| _    { comment lexbuf }